DAT  fileName byte "PanicButtonsBase110714a110721k", 0
CON{{ 

  Modified from Nordic_nRF24L01_DemoMaster110629a

  ---- Private Notes -------
  110710a see notes in header file.
  10c Change the way rx and tx addresses are stored.  Change Nordic driver
  to use multiple targets.
  13a Even though there are multiple targets, they all need to use the
  same address so they will receive the transmissions from the Nordic
  FOBs.
  How is the communication going to be passed from unit to unit.
  Imagine FOB is pressed.  It tx to all units 2, 4 & 6 receive.
  The IRQ should start a timer.  Each uniti knows it has to wait
  a certain about of time before retransmitting signal.
  The tx will have an ID attached.  The ID will include the originating
  unit unless the OU is a FOB then it will ID itself differently.
  I'll probably need some sort of syncronized clock.
  Add the ability to change clock modes with Clock object.
  13c does not receive FOB tx but it does receive tx from
  PanicButtonDebug110713b.
  14a Change config information.  FOB use single byte CRC.
  I also had to change the packet size to four bytes.
  14a works with the Nordic FOB from SparkFun.

  110714a110721a works.  21b still works. 21c works.
  21d seems to work okay.
  21e works okay with three units but not with four.
  21f nextSlot is zero for all units (maybe).
  21g I think I fixed the nextSlot problem.  Now some of the
  units will stop working.
  21i add restarting of Nordic module and move pins above touchpad
  pins.
  21k try clearing clearable bits in status.
  21l try powering down and then restarting.
  
  ---- Public Notes -------
  
  Version 110714a is only receives.  It could be easily modified
  to transmit too.  This object received the four byte payload from
  the Nordic FOB from SparkFun Electronics.  The firmware of the
  FOB had not been modified.
  The FOB transmits four bytes.  The first byte indicates which
  button was pressed.  The "noon" button (nearest the hole to
  attach it to a keychain) causes the FOB to tx $1E.  The "3 o'clock"
  button is indicated by $17 in the first byte of the payload.
  "6 o'clock" is $1D, "9 o'clock" is $1F and the center button
  is $0F.  The button can easily transmit twice per press.
  The second byte is the high byte of the button press count.
  The third byte is the low byte of the count.  The fourth and
  final byte is $00.
  
}} 
CON
  _clkmode = xtal1 + pll16x                           
  _xinfreq = 5_000_000


    
CON

  _Fast = xtal1 + pll16x
  _Slow = RCSLOW
  _CurrentCrystal = 5_000_000

  'Pins to Nordic nRF24L01+
  {_SpiIrq = Header#_DefaultSpiIrq '14
  _SpiMiso = Header#_DefaultSpiMiso '12
  _SpiMosi = Header#_DefaultSpiMosi '10
  _SpiSck = Header#_DefaultSpiSck '8 
  _SpiCsn = Header#_DefaultSpiCsn '6     
  _SpiCe = Header#_DefaultSpiCe '4   

   'Pins to Hong Kong Nordic nRF24L01+
  _SpiIrq = 0
  _SpiMiso = 1
  _SpiMosi = 2
  _SpiSck = 3 
  _SpiCsn = 4     
  _SpiCe = 5    }

  _RxLed = 23   ' set high to turn LED on
  _TxLed = 16   ' set high to turn LED on 

  _ClearableBits = %01110000
  
VAR

  long firmwareVersion
  long tempLong[2]

  byte buttonId ', otherButtonId[Header#_ButtonUnitsUsed]
  byte firmwareSubVersion
  byte payloadSize
  byte globalPayload[Header#_LargestPayloadSize + 1] ' Add 1 so a terminating zero may be added if desired. 
  byte bytesToAdd[2]
  byte debugMode, unitMode, originatingUnit
  byte tempbyte, nibble[8]

  'base station only ?
  byte firmwareStatus[Header#_ButtonUnitsUsed + 1]
  ' Keep below in order
  byte spiIrq,  spiMiso, spiMosi, spiSck, spiCsn, spiCe
  byte nordicModuleType
  ' Keep above in order
  byte aCoupleOfExtraBytesPadding[8]
  
OBJ   

  Debug : "FullDuplexSerial" 'Plus"   
  Nordic : "NordicBeta110721a" '"NordicPlusMultiTarget110713a"      
  Header : "PanicHeader110720b"
  Eeprom : "Eeprom110713a"
  Clock : "Clock"
  
PUB Main | localIndex, timeCnt[2], timeMs, nextSlot, rxTime, txTime, txInterval, {
         } timeSlot, irqMask, timeSinceLastRx, newFreq[2], loopCount, wrongStatusCount 


  Clock.Init(_CurrentCrystal)
  
  Debug.start(31, 30, 0, 57600)

  if firmwareVersion == 0
    ResetSystemAfterFirmwareUpdate

  txInterval := Header#_DelayComInterval *  (Header#_ButtonUnitsUsed + 1)
  timeSlot := Header#_DelayComInterval *  buttonId 
  
  'waitcnt(clkfreq * 2 + cnt)
  Debug.str(String("Start of Nordic Stress Reduction Buttons", 13))
  
  dira[_RxLed]~~                ' ** just use LEDs while debugging
  dira[_TxLed]~~                ' ** remember to comment out later

  dira[spiIrq]~

  'Nordic.Start(_SpiSck, _SpiMiso, _SpiMosi, _SpiCsn, _SpiCe, @config)', @nordicStatus, @address)
  Nordic.Start(spiSck, spiMiso, spiMosi, spiCsn, spiCe, @config)
  payloadSize := byte[@config + Header#_PayloadOffset] ' change whenever Nordic is configured. 
  
  Debug.str(String(13, "Nordic Started", 13))
  
  Debug.str(String(13, "****** buttonId = "))
  Debug.dec(buttonId)
  Debug.str(String(" ******", 13))
  {
  repeat while debug.rxcheck == -1
    Debug.str(String(13, "Press any key to start."))
    waitcnt(clkfreq / 10 + cnt)  }
  'SpiCheck

  '**if firmwareVersion == 0
    ' set lower EEPROM to agree with upper EEPROM
    '**ResetSystemAfterFirmwareUpdate
    
  '** if buttonId == Header#_BaseNordic
    ' Check EEPROM to make sure all the units have uptodate fw
    
    '** not yet CheckFirmwareStatus
    
  'repeat localTargetDevice from 0 to Header#_ButtonUnitsUsed
  {
  Debug.str(String("rx address ="))
  repeat localIndex from 0 to 4
    Debug.str(String(" $"))
    Debug.hex(byte[@rxAddress + localIndex], 2)
  Debug.str(String(13, "tx address ="))
  repeat localIndex from 5 to 9
    Debug.str(String(" $"))
    Debug.hex(byte[@rxAddress + localIndex], 2)
   }
  
 
  'Debug.str(String("Receiver configured. Waiting for packets...", 13))
  Debug.str(String("Receiver configured.", 13))
  Debug.str(String(13, "Transceiver configured for a four byte payload."))
  'Debug.str(String("Receiving only.", 13))
  Debug.str(String(13, "For now the Prop is running at 80MHz.", 13))
  unitMode := Header#_ListenForFobMode 
  'newFreq[0] := Clock.SetClock(_Slow)
  loopCount := 0
  wrongStatusCount := 0  
   
  repeat
    'PreparePacket
    'outa[_TxLed]~~ 
    'Nordic.TransmitString(@packetTemplate)
    'outa[_TxLed]~
    'waitcnt(clkfreq / 2 + cnt) ' give slave device chance to receive and transmit
    if loopCount++ // 100_000 == 0
      DisplayUnitMode
      Debug.str(String(13, "Loop Status = %"))
      localIndex := Nordic.Status
      Debug.bin(localIndex, 8)
      if localIndex == 0
        Debug.str(String(13, "Powering down device."))
        Nordic.SpiWritePub(Nordic#CONFIG, byte[@config] - %10, 1)
        waitcnt(clkfreq * 2 + cnt)
        Debug.str(String(13, "Configuring device."))
        Nordic.Configure(@config)
        Debug.str(String(13, "loop config status = %"))
        localIndex := Nordic.Status
        Debug.bin(localIndex, 8)
        SpiCheck
        Debug.str(String(13, "Configuring device."))
        Nordic.Configure(@config)
        Debug.str(String(13, "post SpiCheck config status = %"))
        
        localIndex := Nordic.Status
        Debug.bin(localIndex, 8)
      elseif localIndex <> %00001110
        repeat
         result := ClearStatus
        while result <> 0
      {if localIndex <> %00101110 and localIndex <> %01100000 and localIndex <> %00001110'nordicStatus    
        'nordicStatus := %00101110
        if wrongStatusCount++ > Header#_WrongStatusLimit
          Debug.str(String(13, "Restarting Nordic device."))
          wrongStatusCount := 0
          Nordic.Start(spiSck, spiMiso, spiMosi, spiCsn, spiCe, @config)
          'payloadSize := byte[@config + Header#_PayloadOffset] ' change whenever Nordic is configured. 
        Debug.str(String(13, "Configuring device."))
        Nordic.Configure(@config)
      else
       wrongStatusCount := 0
      }
    if unitMode == Header#_Ready1stTxMode
      outa[_TxLed]~~
      Debug.str(String(13, "Waiting for 1st tx slot.", 13))
      'nextSlot := (originatingUnit - buttonId) * Header#_DelayComInterval + timeSlot
      nextSlot := (buttonId * Header#_DelayComInterval) '+ rxTime
      Debug.str(String(13, "nS="))
      Debug.dec(nextSlot) 
      repeat while cnt - rxTime > nextSlot - Header#_LoopTimeFudge'(buttonId * Header#_DelayComInterval)'(nextSlot + Header#_LoopTimeFudge)
        nextSlot += txInterval
        Debug.str(String(13, "+nS="))
        Debug.dec(nextSlot)
      Debug.str(String(13, "cnt="))
      Debug.dec(cnt)  
      waitcnt(nextSlot + rxTime )
      Nordic.TransmitString(@packet)
      Debug.str(String(13, "tx packet = "))
      SafeBug(@packet, payloadSize)
      Debug.str(String(13, "rxTime = "))
      Debug.dec(rxTime) 
      Debug.str(String(13, "rxTime + nextSlot = "))
      Debug.dec(rxTime + nextSlot) 
      unitMode := Header#_AlertMode
      outa[_TxLed]~ 
      localIndex := ina[31..0]
      Debug.str(String(13, "ina[31..0] = %"))
      Debug.bin(localIndex, 32)
    if unitMode == Header#_ReadyToTxMode
      outa[_TxLed]~~
      Debug.str(String(13, "Waiting for tx slot.", 13))
      'nextSlot := (originatingUnit - buttonId) * Header#_DelayComInterval + timeSlot
      nextSlot := (buttonId - originatingUnit) * Header#_DelayComInterval' + timeSlot
      Debug.str(String(13, "nS="))
      Debug.dec(nextSlot) 
      repeat while cnt - rxTime > (nextSlot - Header#_LoopTimeFudge)
        nextSlot += txInterval
        Debug.str(String(13, "+nS="))
        Debug.dec(nextSlot)
      Debug.str(String(13, "cnt="))
      Debug.dec(cnt)  
      waitcnt(nextSlot + rxTime)
      Nordic.TransmitString(@packet)
      Debug.str(String(13, "tx packet = "))
      SafeBug(@packet, payloadSize)
      Debug.str(String(13, "rxTime = "))
      Debug.dec(rxTime) 
      Debug.str(String(13, "rxTime + nextSlot = "))
      Debug.dec(rxTime + nextSlot) 
      unitMode := Header#_AlertMode
      outa[_TxLed]~         
    
    if ina[spiIrq]  == 0 'Wait for incoming data
      'We have some data
      rxTime := cnt
      
      Debug.str(String(13, "Status = %"))
      localIndex := Nordic.Status
      Debug.bin(localIndex, 8)     
      'outa[_RxLed]~~
      'Nordic.ReadPayload(@globalPayload)
      ReceivePayload
      'Debug.str(String(13, "Received: "))
      'SafeBug(@globalPayload, payloadSize) 
      'outa[_RxLed]~
      DisplayUnitMode
      repeat
       result := ClearStatus
      while result <> 0
      
PRI ClearStatus

  result := Nordic.Status
  Debug.str(String(13, "ClearStatus method"))
  Debug.str(String(13, "Status = %"))
  Debug.bin(result, 8)
  result &= _ClearableBits
  if result > 0
    Debug.str(String(13, "Writing Status = %"))
    Debug.bin(result, 8)
    Nordic.SpiWritePub(Nordic#_STATUS, @result, 1)  
         
PRI ReceivePayload | actionCode
    
  outa[_RxLed]~~
  Nordic.ReadPayload(@globalPayload)
  Debug.str(String(13, "Received: "))
  SafeBug(@globalPayload, payloadSize)
  if globalPayload[0] > $1F
    NibbleTheBytes(@nibble, @globalPayload, payloadSize * 2)
    actionCode := NibbleAction
    case actionCode
      -1:
        Debug.str(String(13, "Something strange happened.  Error."))
      0:
        Debug.str(String(13, "This unit has already sent this message."))
      other:
        Debug.str(String(13, "Preparing to resend message."))
        Debug.str(String(13, "This message passed through "))
        Debug.dec(actionCode)
        Debug.str(String(" units."))
        ByteOfNibbles(@nibble, @packet, payloadSize * 2)
        originatingUnit := nibble[2]
        unitMode := Header#_ReadyToTxMode
  else
    bytemove(@packet, @packetTemplate, payloadSize)
    packet[0] := Header#_FobPanicTx << 4
    packet[0] += buttonId
    originatingUnit := buttonId
    unitMode := Header#_Ready1stTxMode
  outa[_RxLed]~
            
PRI NibbleAction | localIndex, localTemp
'' Called only to pass along messages.

  Debug.str(String(13, "NibbleAction method ")) 
  localTemp := (payloadSize * 2) - 1
  repeat localIndex[0] from 1 to localTemp
    if nibble[localIndex] == buttonId
      unitMode := Header#_AlertMode
      return 0 ' This unit has already sent this message.
    elseif nibble[localIndex] == Header#_EmptyNibble
      bytemove(@nibble + 2, @nibble + 1, localIndex)
      nibble[1] := buttonId
      unitMode := Header#_ReadyToTxMode
      return localIndex
  NonFatalError(0)
  result := -1
      
PUB NibbleTheBytes(nibblePtr, bytePtr, nibbles) 

  result := nibbles / 2
  Debug.str(String(13, "NibbleTheBytes method "))
       
  repeat result
    byte[nibblePtr++] := byte[bytePtr] >> 4
    Debug.str(String(13, "byte = $"))
    Debug.hex(byte[bytePtr], 2)
    Debug.str(String(", nibbles = $"))
    Debug.hex(byte[nibblePtr - 1], 2)    
    Debug.str(String(", $"))
    byte[nibblePtr++] := byte[bytePtr++] & %00001111
    Debug.hex(byte[nibblePtr - 1], 2)
  if nibbles // 2 == 1
    byte[nibblePtr] := byte[bytePtr] >> 4
    Debug.str(String(13, "byte = $"))
    Debug.hex(byte[bytePtr], 2)
    Debug.str(String(", nibble = $"))
    Debug.hex(byte[nibblePtr], 2)
    
PUB ByteOfNibbles(nibblePtr, bytePtr, nibbles) 

  result := nibbles / 2
  Debug.str(String(13, "ByteOfNibbles method "))
  
  repeat result
    byte[bytePtr] := byte[nibblePtr++] << 4    
    byte[bytePtr++] += byte[nibblePtr++]
    Debug.str(String(13, "byte = $"))
    Debug.hex(byte[bytePtr - 1], 2)
    Debug.str(String(", nibbles = $"))    
    Debug.hex(byte[nibblePtr - 2], 2)
    Debug.str(String(", $"))
    Debug.hex(byte[nibblePtr - 1], 2)
         
  if nibbles // 2 == 1
    byte[bytePtr] := byte[nibblePtr] << 4
    Debug.str(String(13, "byte = $"))
    Debug.hex(byte[bytePtr], 2)
    Debug.str(String(", nibble = $"))    
    Debug.hex(byte[nibblePtr], 2)
    
PRI ResetSystemAfterFirmwareUpdate '| localIndex, localOtherButtonIndex
'' If this method is called, it is because the Prop
'' has just been loaded with this program and has not
''  run the program since it was loaded.
'' This method adjusts the tx and rx addresses and other varibles
'' that are dependent on the unit's ID and firmware version.   

  waitcnt(clkfreq * 2 + cnt)
  Eeprom.WriteProtectChange(0)
  Debug.str(String(13, "ResetSystemAfterFirmwareUpdate"))       
  Eeprom.ToRam(@buttonId, @buttonId, Header#_IdEepromLocation) ' find out unit's ID.

  Debug.str(String(13, "Setting variables for Button Unit #"))
  Debug.dec(buttonId)
  ' set firmware variables in RAM
  firmwareVersion := Header#_FirmwareVersion
  firmwareSubVersion := Header#_FirmwareSubVersion
  Debug.str(String(13, "Firmware Version = "))
  Debug.dec(firmwareVersion)
  Debug.tx(firmwareSubVersion)
  Debug.str(String(13, "Writing firmware version information to EEPROM"))
  
  ' remember firmware and ID even if turned off
  Eeprom.FromRam(@buttonId, @buttonId, @buttonId) ' keep button ID in lower EEPROM too 
  Eeprom.FromRam(@firmwareVersion, @firmwareVersion + 3, @firmwareVersion) 
  Eeprom.FromRam(@firmwareSubVersion, @firmwareSubVersion, @firmwareSubVersion) 

  ' check upper EEPROM for firmware info
  Eeprom.ToRam(@tempLong, @tempLong + 3, Header#_FirmwareLocation)
  Eeprom.ToRam(@tempbyte, @tempbyte, Header#_FirmwareSubLocation)

  if tempLong <> Header#_FirmwareVersion or tempByte <> Header#_FirmwareSubVersion
    ' Update upper EEPROM with correct firmware info 
    Eeprom.FromRam(@firmwareVersion, @firmwareVersion + 3, @firmwareVersion) 
    Eeprom.FromRam(@firmwareSubVersion, @firmwareSubVersion, @firmwareSubVersion) 

  case buttonID
    0: nordicModuleType := Header#_NordicTypeUnit00
    1: nordicModuleType := Header#_NordicTypeUnit01
    2: nordicModuleType := Header#_NordicTypeUnit02
    3: nordicModuleType := Header#_NordicTypeUnit03
    4: nordicModuleType := Header#_NordicTypeUnit04
    5: nordicModuleType := Header#_NordicTypeUnit05
    6: nordicModuleType := Header#_NordicTypeUnit06 
    other: FatalError(0)
    
  'Pins to Nordic nRF24L01+
  Debug.str(String(13, 13, "****************************************")) 
  Debug.str(String("****************************************", 13)) 
  Debug.str(String(13, "  This button unit uses a ")) 
  if nordicModuleType == Header#_SparkFunModule
    'Pins to SparkFun Nordic nRF24L01+ modules
    Debug.str(String("SparkFun"))
    spiIrq := Header#_SfeSpiIrq 
    spiMiso := Header#_SfeSpiMiso 
    spiMosi := Header#_SfeSpiMosi 
    spiSck := Header#_SfeSpiSck 
    spiCsn := Header#_SfeSpiCsn     
    spiCe := Header#_SfeSpiCe 
    
  elseif nordicModuleType == Header#_HongKongModule
    'Pins to Hong Kong Nordic nRF24L01+ modules
    Debug.str(String("Hong Kong"))
    spiIrq := Header#_HkSpiIrq 
    spiMiso := Header#_HkSpiMiso 
    spiMosi := Header#_HkSpiMosi 
    spiSck := Header#_HkSpiSck 
    spiCsn := Header#_HkSpiCsn     
    spiCe := Header#_HkSpiCe 
    
  else
    FatalError(1)
  Debug.str(String(" Nordic module.")) 
  Debug.str(String(13, 13, "****************************************")) 
  Debug.str(String("****************************************", 13)) 
  Eeprom.FromRam(@spiIrq, @spiIrq + (@nordicModuleType - @spiIrq) + 1, @spiIrq)  
                        ' @spiIrq + 6
  Eeprom.WriteProtectChange(1)
  waitcnt(clkfreq * 2 + cnt)   
    
PRI CheckFirmwareStatus | localIndex
'' This method will probably only be used by the basestation.
'' This method checks to see if the firmware of the other button
'' units has been successfully updated since the basestation's
'' fireware was updated.
' read EEPROM and check fw versions and status codes

  repeat localIndex from Header#_PinkNordic to Header#_VioletNordic
    if firmwareStatus[localIndex] == Header#_NoInfoYetFw
      Eeprom.ToRam(@firmwareStatus, @firmwareStatus + Header#_VioletNordic, {
      } Header#_FirmwareLocation - (localIndex * Header#_FirmwareDiscriptionSize) + 5)
      ' also need to check install status of firmware
      
PUB PreparePacket | localIndex

  Hex(bytesToAdd[0]++, 2, @packetTemplate + 9)
  bytesToAdd[1]++ ' The second byte will increament by 2
  Hex(bytesToAdd[1]++, 2, @packetTemplate + 15)

PRI NonFatalError(errorCode)
'' Used to show errors that don't stop opperation of unit
' add piezo alarm
' add external alarm LED
  Debug.str(String(13, "Error Code = "))
  Debug.dec(errorCode)
  
  dira[23..15]~~
  if errorCode => 0 and errorCode =< 7
    result := 1
    result <<= errorCode
    repeat 12 ' magic number?
      outa[23..15] := %11111111 - result
      waitcnt(clkfreq / 8 + cnt)
      outa[23..15] := 0
      waitcnt(clkfreq / 8 + cnt)
  else
    repeat 12 ' magic number? 
      outa[23..15] := %11111111
      waitcnt(clkfreq / 8 + cnt)
      outa[23..15] := 0
      waitcnt(clkfreq / 8 + cnt)
      
PRI FatalError(errorCode)
'' No exit from this method
'' Used to show fatal errors
' add piezo alarm
' add external alarm LED
  Debug.str(String(13, "Fatal Error Code = "))
  Debug.dec(errorCode)
  
  dira[23..15]~~
  if errorCode => 0 and errorCode =< 7
    result := 1
    result <<= errorCode
    repeat 
      outa[23..15] := %11111111 - result
      waitcnt(clkfreq / 8 + cnt)
      outa[23..15] := 0
      waitcnt(clkfreq / 8 + cnt)
  else
    repeat 
      outa[23..15] := %11111111
      waitcnt(clkfreq / 8 + cnt)
      outa[23..15] := 0
      waitcnt(clkfreq / 8 + cnt)

PUB hex(value, digits, localPtr)
'' ripped from FullDuplexSerial
  value <<= (8 - digits) << 2
  repeat digits
    byte[localPtr++] := lookupz((value <-= 4) & $F : "0".."9", "A".."F")
      
PUB FromAsciiHex(digits, localPtr) | localCharacter  
'' not ripped hence suboptimal
  repeat digits
    localCharacter := byte[localPtr++]
    if localCharacter => "0" and localCharacter =< "9"
      localCharacter -= "0"
    elseif localCharacter => "A" and localCharacter =< "F"
      localCharacter -= "A"
      localCharacter += $0A
    else
      localCharacter := 0
    result <<= 4
    result += localCharacter

PUB SpiCheck | localIndex

  Debug.str(String(13, "Attempting to read rx address from module."))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  Debug.str(String(13, "rx address = $"))
  repeat localIndex from 0 to 4
    Debug.hex(globalPayload[localIndex], 2)
  if CompareArrays(@globalPayload, @testAddress, 5) == -1
    Debug.str(String(13, "The present rx address is the same as the test address."))
    Debug.str(String(13, "Changing the test address from $"))
    repeat localIndex from 0 to 4
      Debug.hex(testAddress[localIndex], 2)
    Debug.str(String(", to $"))
    repeat localIndex from 0 to 4
      testAddress[localIndex] += $11
      Debug.hex(testAddress[localIndex], 2)
  Debug.str(String(13, "Attempting to write following rx address to module: $"))
  repeat localIndex from 0 to 4
    Debug.hex(testAddress[localIndex], 2)
  Nordic.SpiWritePub(Nordic#RX_ADDR_P0, @testAddress, 5)
  Debug.str(String(13, "Reading rx address from module: $"))
  Nordic.SpiReadPub(Nordic#RX_ADDR_P0, @globalPayload, 5)
  repeat localIndex from 0 to 4
    Debug.hex(testAddress[localIndex], 2)
  if CompareArrays(@globalPayload, @testAddress, 5) == -1
    Debug.str(String(13, 13, "!!!!Success!!!!")) 
    Debug.str(String(13, 13, "The rx address was successfully changed.")) 
    Debug.str(String(13, "The test rx address will not necessarily be the rx address used")) 
    Debug.str(String(13, "in communicating with other modules.", 13)) 
  else
    Debug.str(String(13, 13, "** Failure **")) 
    Debug.str(String(13, 13, "The rx address was NOT changed.")) 
    Debug.str(String(13, "The Propeller was not able to communicate with the")) 
    Debug.str(String(13, "Nordic module.", 13)) 
  
PRI CompareArrays(localPtr0, localPtr1, localSize)

  result := -1
  repeat localSize
    if byte[localPtr0++] <> byte[localPtr1++]
      result := 0
        
PRI DisplayUnitMode

  Debug.str(String(13, "unitMode = "))
  Debug.dec(unitMode)
  Debug.str(String(", = ", 34))
  case unitMode
    Header#_ListenForFobMode:
      Debug.str(String("_ListenForFobMode"))
    Header#_AlertMode:
      Debug.str(String("_AlertMode"))
    Header#_ReadyToTxMode:
      Debug.str(String("_ReadyToTxMode"))
    Header#_PowerDownPleaseMode:
      Debug.str(String("_PowerDownPleaseMode"))
    Header#_Ready1stTxMode:
      Debug.str(String("_Ready1stTxMode"))

PUB SafeBug(localPtr, localSize)

  repeat result from 0 to localSize - 1
    TbugTx(byte[localPtr + result])
      
PRI TbugTx(localCharacter)

  if (localCharacter > 31 and localCharacter < 127) or localCharacter == 13
    Debug.tx(localCharacter)
  else
    Debug.tx(60)
    Debug.tx(36)
    Debug.hex(localCharacter, 2)
    Debug.tx(62)    

DAT

                                       '1         2         3
                             '01234567890123456789012345678901
'packetTemplate          byte "what is $00 + $00?", 13, 0[14]
packetTemplate          byte $FF[4], 0
packet                  byte $FF[4], 0

testAddress   byte $AA, $BB, $CC, $DD, $EE


''------------------ Cofig settings for new nRF24L01+ modules ----------------------------------------
'' ----------------- Keep below together and in order ------------------------------------------------
config        byte Header#_ListenForFobReg0, Header#_ListenForFobReg1, Header#_ListenForFobReg2
              byte Header#_ListenForFobReg3, Header#_ListenForFobReg4, Header#_ListenForFobReg5
              byte Header#_ListenForFobReg6
nordicStatus  byte Header#_NordicStatus
rxPayload     byte Header#_ListenForFobRxPayload '_RxPayload '25 '32   ' Register $11 RX_PW_P0

rxAddress     byte Header#_InitialAddressCharacter[5] ' Register $10 Transmit address data pipe 0, LSByte first
txAddress     byte Header#_InitialAddressCharacter[5] ' Register $10 Transmit address data pipe 0, LSByte first
'' ----------------- Keep above together and in order ------------------------------------------------             